#include "View.h"
#include <QEvent>
#include <qmath.h>
#include <QTimer>
#include <QCoreApplication>

const float MinOpacity = 0.01f;

View::View(QWidget *parent) :
        QGraphicsView(parent)
{
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    counter = 0;
    loopIsRunning = false;
    exploding = false;

    setBackgroundBrush(QBrush(QColor(64, 187, 201)));

    setScene(&scene);
    setSceneRect(-400, -200, 400, 200);

    // I'm highly skeptical on the behaviour of centerOn too!
    // When I make the SceneRect larger it appears to "center"
    // the chosen point or grapics item in the bottom right corner.
    // This isn't VG backend specific, it happens in the Qt simulator.
    centerOn(300, 200);
}

void View::loop()
{
    while(loopIsRunning)
    {
        updatePositions();
        QCoreApplication::processEvents();
    }
}

bool View::viewportEvent(QEvent *event)
{
    switch (event->type()) {
    case QEvent::Show:
    case QEvent::Enter: // needed by Maemo
    case QEvent::WindowActivate: // needed by Symbian
        loopIsRunning = true;
        QTimer::singleShot(100, this, SLOT(loop()));
        break;
    case QEvent::Leave: // needed by Maemo
    case QEvent::WindowDeactivate: // needed by Symbian
        loopIsRunning = false;
        break;
    default:
        break;
    }

    return QGraphicsView::viewportEvent(event);
}

void View::updatePositions()
{
    counter++;
    if (exploding)
    {
        if (!particleUpdateStep())
        {
            exploding = false;
        }
    }
    else if ((counter % 120) == 0)
    {
        exploding = true;
        particleSpawn(QPointF((qrand() % 200) - 100, (qrand() %100) - 50));
    }
    else
    {
        // Make sure we always block on a redraw before looping again?
        update();
    }
}

void View::particleSpawn(QPointF position)
{
    ParticleItem *particle;

    for (int i=0; i<=20; i++)
    {
        particle = new ParticleItem(QColor(255, 69, 0));
        scene.addItem(particle);
        particle->setPos(position);
        particle->setOpacity(1.0f);
        qreal rot = qrand() % 360;
        particle->setRotation(rot + 90);
        rot = rot * (M_PI / 180);

        qreal speed = 5;//(qrand() % 5) + 7;
        particle->xVel = speed * qCos(rot);
        particle->yVel = speed * qSin(rot);
        particles.append(particle);
    }
}

bool View::particleUpdateStep()
{
    bool anotherStep = true;
    ParticleItem *particle;

    for (int i = 0; i < particles.size(); i++)
    {
        particle = particles.at(i);
        particle->setPos(particle->x() + particle->xVel, particle->y() + particle->yVel);
        //particle->xVel *= 0.9f;
        //particle->yVel *= 0.9f;
        //particle->setOpacity(particle->opacity() * 0.95f);
        if (counter % 120 > 90) //(particle->opacity() < MinOpacity)
        {
            anotherStep = false;
            scene.removeItem(particle);
            delete particle;
        }
    }

    if (!anotherStep)
    {
        particles.clear();
    }

    return anotherStep;
}
